'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.storage = exports.metadata = exports.auth = undefined;

var _nodeFetch = require('node-fetch');

var _nodeFetch2 = _interopRequireDefault(_nodeFetch);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Authorization adapter
var auth = exports.auth = function auth(req, options, data, cb) {
	if (!options.fetchUser) return cb();

	if (options.byToken === false) {
		return cb('Auth error: Only token authentication is supported.');
	}

	(0, _nodeFetch2.default)(options.config.host + '/users', {
		headers: {
			'Content-Type': 'application/json',
			'Authorization': 'Token ' + req.auth.token
		},
		timeout: options.timeout
	}).catch(function (err) {
		return cb('Auth error: ' + err.message);
	}).then(function (res) {
		if (res.status >= 200 && res.status < 300) return res.json();

		res.json().then(function (data) {
			return cb('Auth error: ' + data.message);
		});
	}).then(function (rows) {
		if (!rows || !rows[0]) return cb('Auth error: No user for token.');

		cb(null, rows[0]);
	}).catch(function (err) {
		return cb('Auth error: ' + err.message);
	});
};

// Metadata adapter
var cache = {};

var metadata = exports.metadata = function metadata(req, options, data, _ref, cb) {
	var SensorAttribute = _ref.SensorAttribute;

	// Check cache
	var id = data.getDeviceId();
	var cached = cache[id];

	if (cached && cached.timestamp > Date.now() - options.cacheExpireTime) {
		data.setInfo(cached.info);
		data.setAttributes(cached.attributes);
		return cb();
	}

	// Query metadata
	var include = 'template.attributes.converters,' + 'template.attributes.calibrators,' + 'template.attributes.validators,' + 'labels';

	(0, _nodeFetch2.default)(options.config.host + '/devices?filter=udid:' + id + '&include=' + include, {
		headers: {
			'Content-Type': 'application/json',
			'Authorization': 'Token ' + req.auth.token
		},
		timeout: options.timeout
	}).catch(function (err) {
		return cb('Metadata error: ' + err.message);
	}).then(function (res) {
		if (res.status >= 200 && res.status < 300) return res.json();

		res.json().then(function (data) {
			return cb('Metadata error: ' + data.message);
		});
	}).then(function (rows) {
		if (!rows[0] || !rows[0].template || !rows[0].template.attributes) {
			return cb('Metadata error: No metadata available for device ' + id);
		}

		// Create attributes
		var attributes = rows[0].template.attributes.map(function (row) {
			var attr = new SensorAttribute(row.name);
			var converters = row.converters;
			var calibrators = row.calibrators;
			var validators = row.validators;


			converters && converters.forEach(function (row) {
				attr.addConverter(row.type, row.value);
			});
			calibrators && calibrators.forEach(function (row) {
				attr.addCalibrator(new Function(row.fn));
			});
			validators && validators.forEach(function (row) {
				attr.addValidator(row.type, row.value);
			});

			return attr;
		});

		// Create info object
		var info = {};
		var labels = rows[0].labels;

		labels && labels.forEach(function (label) {
			info[label.key] = label.value;
		});

		info._id = id;

		// Cache result
		cache[id] = { info: info, attributes: attributes, timestamp: Date.now() };

		// Done
		data.setInfo(info);
		data.setAttributes(attributes);
		cb();
	}).catch(function (err) {
		return cb('Metadata error: ' + err.message);
	});
};

// Storage adapter
var storage = exports.storage = function storage(req, options, data, cb) {
	var id = data.getInfo()._id;

	(0, _nodeFetch2.default)(options.config.host + '/devices/' + id + '/measurements', {
		method: 'POST',
		headers: {
			'Content-Type': 'application/json',
			'Authorization': 'Token ' + req.auth.token
		},
		body: JSON.stringify(data.getData()),
		timeout: options.timeout
	}).catch(function (err) {
		return cb('Storage error: ' + err.message);
	}).then(function (res) {
		if (res.status >= 200 && res.status < 300) return cb();

		res.json().then(function (data) {
			return cb('Storage error: ' + data.message);
		});
	}).catch(function (err) {
		return cb('Storage error: ' + err.message);
	});
};